from zope.interface import implements
from zope.component import adapts
from zope.i18nmessageid import MessageFactory
from archetypes.schemaextender.interfaces import (
    IOrderableSchemaExtender,
    IBrowserLayerAwareExtender,
)
from archetypes.schemaextender.field import ExtensionField
from Products.Archetypes.utils import OrderedDict
from Products.Archetypes.public import (
    ImageField,
    ImageWidget,
)
from Products.Archetypes.interfaces import IBaseObject
from collective.contextimage.interfaces import (
    IPageImageExtensionLayer,
    IHeaderImageExtensionLayer,
    IViewletImageExtensionLayer,
    ILogoImageExtensionLayer,
)

_ = MessageFactory('collective.contextimage')


class XImageField(ExtensionField, ImageField):
    pass


class ExtenderBase(object):
    
    implements(IOrderableSchemaExtender, IBrowserLayerAwareExtender)
    adapts(IBaseObject)
    
    def __init__(self, context):
        self.context = context

    def getFields(self):
        return self.fields
    
    def getOrder(self, original):
        neworder = OrderedDict()
        keys = original.keys()
        last = keys.pop()
        keys.insert(1, last)
        for schemata in keys:
            neworder[schemata] = original[schemata]
        return neworder


class PageContextImageExtender(ExtenderBase):
    """Schema extender for context specific images displayed as background
    image of portal.
    """
    
    layer = IPageImageExtensionLayer

    fields = [
        XImageField(
            name='page_context_image',
            schemata='Context Image',
            widget=ImageWidget(
                label=_(u'label_page_context_image', u'Page Background Image'),
            )
        ),
    ]


class HeaderContextImageExtender(ExtenderBase):
    """Schema extender for context specific images displayed as background
    image of portal header.
    """
    
    layer = IHeaderImageExtensionLayer

    fields = [
        XImageField(
            name='header_context_image',
            schemata='Context Image',
            widget=ImageWidget(
                label=_(u'label_header_context_image',
                        u'Header Background Image'),
            )
        ),
    ]


class ViewletContextImageExtender(ExtenderBase):
    """Schema extender for context specific images displayed as viewlet.
    """
    
    layer = IViewletImageExtensionLayer

    fields = [
        XImageField(
            name='viewlet_context_image',
            schemata='Context Image',
            widget=ImageWidget(
                label=_(u'label_viewlet_context_image', u'Viewlet Image'),
            )
        ),
    ]

class LogoContextImageExtender(ExtenderBase):
    """Schema extender for context specific images displayed as logo.
    """

    layer = ILogoImageExtensionLayer

    fields = [
        XImageField(
            name='logo_context_image',
            schemata='Logo Image',
            widget=ImageWidget(
                label=_(u'label_viewlet_logo_image', u'Logo Image'),
            )
        ),
    ]



# -*- coding: utf-8 -*-
import logging
from plone.app.layout.viewlets.common import (
    ViewletBase,
    LogoViewlet,
)
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Acquisition import aq_inner, aq_parent

logger = logging.getLogger('collective.contextimage')

class ContextImageBase(object):
    """Abstract Base for Context Images"""
    image_field_name = None
    default_imageurl = None

    def aquire_context_image(self):
        obj = aq_inner(self.context)
        image = None
        while not IPloneSiteRoot.providedBy(obj):
            try:
                field = obj.getField(self.image_field_name)
            except AttributeError, e:
                return
            if field is None:
                obj = aq_parent(aq_inner(obj))
                continue
            image = field.get(obj)
            if not image:
                obj = aq_parent(aq_inner(obj))
            else:
                break
        return image

    @property
    def imageurl(self):
        image = self.aquire_context_image()
        if not image:
            return '%s/%s' % (self.context.absolute_url(),
                              self.default_imageurl)
        return image.absolute_url()


class ImageViewlet(ContextImageBase, ViewletBase):
    """Base Viewlet for Context Images"""


class CSSViewlet(ImageViewlet):

    default_css_template = None

    @property
    def css(self):
        try:
            return self.default_css_template % self.imageurl
        except Exception, e:
            logger.error(str(e))
            return u''


PAGE_CSS_TEMPLATE = """
html {
    background:url('%s') no-repeat top center fixed;
    -webkit-background-size:cover;
    -moz-background-size:cover;
    -o-background-size:cover;
    background-size:cover;
}
"""

PAGE_DEFAULT_IMAGE = '++resource++collective.contextimage.images/default_page.png'


class PageImageViewlet(CSSViewlet):
    image_field_name = 'page_context_image'

    @property
    def default_imageurl(self):
        return PAGE_DEFAULT_IMAGE

    @property
    def default_css_template(self):
        return PAGE_CSS_TEMPLATE


HEADER_CSS_TEMPLATE = """
#portal-header {
    background: url('%s') no-repeat 0px 0px !important;
}
"""

HEADER_DEFAULT_IMAGE = '++resource++collective.contextimage.images/default.png'


class HeaderImageViewlet(CSSViewlet):
    image_field_name = 'header_context_image'

    @property
    def default_imageurl(self):
        return HEADER_DEFAULT_IMAGE

    @property
    def default_css_template(self):
        return HEADER_CSS_TEMPLATE


class ContextImageViewlet(ImageViewlet):

    @property
    def image(self):
        default = '<img src="%s/viewlet_context_image_default.png" ' + \
                  'alt="Context specific Image" />'
        default = default % self.context.absolute_url()
        image = self.aquire_context_image()
        if not image:
            return default
        try:
            return image.tag()
        except Exception, e:
            logger.error(str(e))
            return default


class ContextLogoViewlet(ContextImageBase, LogoViewlet):
    image_field_name = 'logo_context_image'

    def update(self):
        super(ContextLogoViewlet, self).update()
        image = self.aquire_context_image()
        if image:
            logoTitle = self.portal_state.portal_title()
            self.logo_tag = image.tag(title=logoTitle, alt=logoTitle)



from zope.interface import Interface


class IContextImageExtensionLayer(Interface):
    """Browser layer for collective.contextimage
    """


class IPageImageExtensionLayer(IContextImageExtensionLayer): pass


class IHeaderImageExtensionLayer(IContextImageExtensionLayer): pass


class IViewletImageExtensionLayer(IContextImageExtensionLayer): pass


class ILogoImageExtensionLayer(IContextImageExtensionLayer): pass



from setuptools import setup, find_packages
import os

version = "1.0.1"
shortdesc = ('Context sensitive images from extended plone content as viewlet, '
             'header, logo, ...')
longdesc = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
longdesc += open(os.path.join(os.path.dirname(__file__), 'HISTORY.rst')).read()
longdesc += open(os.path.join(os.path.dirname(__file__), 'LICENSE.rst')).read()

setup(name='collective.contextimage',
      version=version,
      description=shortdesc,
      long_description=longdesc,
      classifiers=[
            'Development Status :: 5 - Production/Stable',
            'License :: OSI Approved :: GNU General Public License (GPL)',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Topic :: Software Development',
            "Framework :: Plone",
            "Framework :: Plone :: 4.1",
            "Framework :: Plone :: 4.2",
      ],
      keywords='web zope plone theme viewlet logo header',
      author='BlueDynamics Alliance',
      author_email='dev@bluedynamics.com',
      url="http://pypi.python.org/pypi/collective.contextimage",
      license='GPL',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'archetypes.schemaextender',
          'plone.app.layout',
      ],
      entry_points="""
      # -*- Entry points: -*-

      [z3c.autoinclude.plugin]
      target = plone
      """,
      )
